@@ -9,7 +9,7 @@ from os.path import expanduser
 import six
 from boto3 import Session
 from localstack.constants import (
-    DEFAULT_SERVICE_PORTS, LOCALHOST, DEFAULT_PORT_WEB_UI, TRUE_STRINGS, FALSE_STRINGS,
+    DEFAULT_SERVICE_PORTS, LOCALHOST, LOCALHOST_IP, DEFAULT_PORT_WEB_UI, TRUE_STRINGS, FALSE_STRINGS,
     DEFAULT_LAMBDA_CONTAINER_REGISTRY, DEFAULT_PORT_EDGE, AWS_REGION_US_EAST_1)
 
 
@@ -49,12 +49,17 @@ KINESIS_ERROR_PROBABILITY = float(os.environ.get('KINESIS_ERROR_PROBABILITY', ''
 
 # randomly inject faults to DynamoDB
 DYNAMODB_ERROR_PROBABILITY = float(os.environ.get('DYNAMODB_ERROR_PROBABILITY', '').strip() or 0.0)
+DYNAMODB_READ_ERROR_PROBABILITY = float(os.environ.get('DYNAMODB_READ_ERROR_PROBABILITY', '').strip() or 0.0)
+DYNAMODB_WRITE_ERROR_PROBABILITY = float(os.environ.get('DYNAMODB_WRITE_ERROR_PROBABILITY', '').strip() or 0.0)
 
 # JAVA EE heap size for dynamodb
 DYNAMODB_HEAP_SIZE = os.environ.get('DYNAMODB_HEAP_SIZE', '').strip() or '256m'
 
 # expose services on a specific host internally
-HOSTNAME = os.environ.get('HOSTNAME', '').strip() or LOCALHOST
+# Note: This used to be os.environ['HOSTNAME'] but since this has caused several issues with hostnames
+# that could not be resolved, we're hardcoding this to 'localhost' (as its purpose is local invocations)
+# TODO: potentially remove this entirely in the future ?!
+HOSTNAME = LOCALHOST
 
 # expose services on a specific host externally
 HOSTNAME_EXTERNAL = os.environ.get('HOSTNAME_EXTERNAL', '').strip() or LOCALHOST
@@ -71,6 +76,9 @@ LAMBDA_REMOTE_DOCKER = is_env_true('LAMBDA_REMOTE_DOCKER')
 # network that the docker lambda container will be joining
 LAMBDA_DOCKER_NETWORK = os.environ.get('LAMBDA_DOCKER_NETWORK', '').strip()
 
+# custom DNS server that the docker lambda container will use
+LAMBDA_DOCKER_DNS = os.environ.get('LAMBDA_DOCKER_DNS', '').strip()
+
 # default container registry for lambda execution images
 LAMBDA_CONTAINER_REGISTRY = os.environ.get('LAMBDA_CONTAINER_REGISTRY', '').strip() or DEFAULT_LAMBDA_CONTAINER_REGISTRY
 
@@ -101,7 +109,7 @@ if TMP_FOLDER.startswith('/var/folders/') and os.path.exists('/private%s' % TMP_
 HOST_TMP_FOLDER = os.environ.get('HOST_TMP_FOLDER', TMP_FOLDER)
 
 # whether to enable verbose debug logging
-DEBUG = os.environ.get('DEBUG', '').lower() in TRUE_STRINGS
+DEBUG = is_env_true('DEBUG')
 
 # whether to use SSL encryption for the services
 USE_SSL = is_env_true('USE_SSL')
@@ -121,6 +129,9 @@ DOCKER_CMD = os.environ.get('DOCKER_CMD', '').strip() or 'docker'
 # whether to start the web API
 START_WEB = os.environ.get('START_WEB', '').strip() not in FALSE_STRINGS
 
+# whether to forward edge requests in-memory (instead of via proxy servers listening on backend ports)
+# TODO: this will likely become the default and may get removed in the future
+FORWARD_EDGE_INMEM = True
 # port number for the edge service, the main entry point for all API invocations
 EDGE_PORT = int(os.environ.get('EDGE_PORT') or 0) or DEFAULT_PORT_EDGE
 # fallback port for non-SSL HTTP edge service (in case HTTPS edge service cannot be used)
@@ -143,6 +154,8 @@ DISABLE_EVENTS = is_env_true('DISABLE_EVENTS')
 # Whether to skip downloading additional infrastructure components (e.g., custom Elasticsearch versions)
 SKIP_INFRA_DOWNLOADS = os.environ.get('SKIP_INFRA_DOWNLOADS', '').strip()
 
+# Adding Stepfunctions default port
+LOCAL_PORT_STEPFUNCTIONS = int(os.environ.get('LOCAL_PORT_STEPFUNCTIONS') or 8083)
 # Stepfunctions lambda endpoint override
 STEPFUNCTIONS_LAMBDA_ENDPOINT = os.environ.get('STEPFUNCTIONS_LAMBDA_ENDPOINT', '').strip()
 
@@ -152,6 +165,9 @@ WINDOWS_DOCKER_MOUNT_PREFIX = os.environ.get('WINDOWS_DOCKER_MOUNT_PREFIX', '/ho
 # whether to use a proxy server with HTTP/2 support. TODO: remove in the future
 USE_HTTP2_SERVER = os.environ.get('USE_HTTP2_SERVER', '').strip() not in FALSE_STRINGS
 
+# name of the main Docker container
+MAIN_CONTAINER_NAME = os.environ.get('MAIN_CONTAINER_NAME', '').strip() or 'localstack_main'
+
 
 def has_docker():
     try:
@@ -167,7 +183,7 @@ def is_linux():
         out = subprocess.check_output('uname -a', shell=True)
         out = out.decode('utf-8') if isinstance(out, six.binary_type) else out
         return 'Linux' in out
-    except subprocess.CalledProcessError:
+    except Exception:
         return False
 
 
@@ -196,7 +212,8 @@ CONFIG_ENV_VARS = ['SERVICES', 'HOSTNAME', 'HOSTNAME_EXTERNAL', 'LOCALSTACK_HOST
                    'WINDOWS_DOCKER_MOUNT_PREFIX', 'USE_HTTP2_SERVER',
                    'SYNCHRONOUS_API_GATEWAY_EVENTS', 'SYNCHRONOUS_KINESIS_EVENTS',
                    'SYNCHRONOUS_SNS_EVENTS', 'SYNCHRONOUS_SQS_EVENTS', 'SYNCHRONOUS_DYNAMODB_EVENTS',
-                   'DYNAMODB_HEAP_SIZE']
+                   'DYNAMODB_HEAP_SIZE', 'MAIN_CONTAINER_NAME', 'LAMBDA_DOCKER_DNS',
+                   'USE_MOTO_CF']
 
 for key, value in six.iteritems(DEFAULT_SERVICE_PORTS):
     clean_key = key.upper().replace('-', '_')
@@ -253,10 +270,12 @@ if is_in_docker and not os.environ.get('LAMBDA_REMOTE_DOCKER', '').strip():
     LAMBDA_REMOTE_DOCKER = True
 
 # local config file path in home directory
-CONFIG_FILE_PATH = os.path.join(expanduser('~'), '.localstack')
+CONFIG_FILE_PATH = os.path.join(TMP_FOLDER, '.localstack')
+if not is_in_docker:
+    CONFIG_FILE_PATH = os.path.join(expanduser('~'), '.localstack')
 
 # set variables no_proxy, i.e., run internal service calls directly
-no_proxy = ','.join(set((LOCALSTACK_HOSTNAME, HOSTNAME, LOCALHOST, '127.0.0.1', '[::1]')))
+no_proxy = ','.join(set((LOCALSTACK_HOSTNAME, HOSTNAME, LOCALHOST, LOCALHOST_IP, '[::1]')))
 if os.environ.get('no_proxy'):
     os.environ['no_proxy'] += ',' + no_proxy
 elif os.environ.get('NO_PROXY'):
@@ -305,6 +324,7 @@ def populate_configs(service_ports=None):
 
     SERVICE_PORTS = service_ports or parse_service_ports()
     globs = globals()
+    protocol = get_protocol()
 
     # define service ports and URLs as environment variables
     for key, value in six.iteritems(DEFAULT_SERVICE_PORTS):
@@ -314,9 +334,13 @@ def populate_configs(service_ports=None):
         port_var_name = 'PORT_%s' % key_upper
         port_number = service_port(key)
         globs[port_var_name] = port_number
-        url = '%s://%s:%s' % (get_protocol(), LOCALSTACK_HOSTNAME, port_number)
+        url = '%s://%s:%s' % (protocol, LOCALSTACK_HOSTNAME, port_number)
         # define TEST_*_URL variables with mock service endpoints
         url_key = 'TEST_%s_URL' % key_upper
+        # allow overwriting TEST_*_URL from user-defined environment variables
+        existing = os.environ.get(url_key)
+        url = existing or url
+        # set global variable
         globs[url_key] = url
         # expose HOST_*_URL variables as environment variables
         os.environ[url_key] = url
@@ -330,6 +354,13 @@ def populate_configs(service_ports=None):
 
 
 def service_port(service_key):
+    if FORWARD_EDGE_INMEM:
+        if service_key == 'elasticsearch':
+            # TODO Elasticsearch domains are a special case - we do not want to route them through
+            # the edge service, as that would require too many route mappings. In the future, we
+            # should integrate them with the port range for external services (4510-4530)
+            return SERVICE_PORTS.get(service_key, 0)
+        return EDGE_PORT_HTTP or EDGE_PORT
     return SERVICE_PORTS.get(service_key, 0)
 
 
@@ -342,6 +373,11 @@ def external_service_url(service_key, host=None):
     return '%s://%s:%s' % (get_protocol(), host, service_port(service_key))
 
 
+def get_edge_url():
+    port = EDGE_PORT_HTTP or EDGE_PORT
+    return '%s://%s:%s' % (get_protocol(), LOCALSTACK_HOSTNAME, port)
+
+
 # initialize config values
 populate_configs()
 
@@ -355,3 +391,6 @@ BUNDLE_API_PROCESSES = True
 
 # whether to use a CPU/memory profiler when running the integration tests
 USE_PROFILER = is_env_true('USE_PROFILER')
+
+# whether to use the legacy CF deployment based on moto (TODO: remove in a future release)
+USE_MOTO_CF = is_env_true('USE_MOTO_CF')
