@@ -2,7 +2,7 @@ import os
 import localstack_client.config
 
 # LocalStack version
-VERSION = '0.12.2'
+VERSION = '0.12.5'
 
 # constant to represent the "local" region, i.e., local machine
 REGION_LOCAL = 'local'
@@ -62,6 +62,7 @@ ENV_INTERNAL_TEST_RUN = 'LOCALSTACK_INTERNAL_TEST_RUN'
 APPLICATION_AMZ_JSON_1_0 = 'application/x-amz-json-1.0'
 APPLICATION_AMZ_JSON_1_1 = 'application/x-amz-json-1.1'
 APPLICATION_AMZ_CBOR_1_1 = 'application/x-amz-cbor-1.1'
+APPLICATION_CBOR = 'application/cbor'
 APPLICATION_JSON = 'application/json'
 APPLICATION_XML = 'application/xml'
 APPLICATION_X_WWW_FORM_URLENCODED = 'application/x-www-form-urlencoded'
@@ -104,6 +105,14 @@ LOCALSTACK_INFRA_PROCESS = 'LOCALSTACK_INFRA_PROCESS'
 
 # hardcoded AWS account ID used by moto
 MOTO_ACCOUNT_ID = TEST_AWS_ACCOUNT_ID
+# fix moto account ID - note: keep this at the top level here
+try:
+    from moto import core as moto_core
+    from moto.core import models as moto_core_models
+    moto_core.ACCOUNT_ID = moto_core_models.ACCOUNT_ID = MOTO_ACCOUNT_ID
+except Exception:
+    # ignore import errors
+    pass
 
 # default AWS region us-east-1
 AWS_REGION_US_EAST_1 = 'us-east-1'
@@ -117,6 +126,10 @@ try:
 except Exception:
     MAX_POOL_CONNECTIONS = 150
 
-# test credentials being used while generating signature for S3 presign url
+# test credentials used for generating signature for S3 presigned URLs (to be used by external clients)
 TEST_AWS_ACCESS_KEY_ID = 'test'
 TEST_AWS_SECRET_ACCESS_KEY = 'test'
+
+# credentials being used for internal calls
+INTERNAL_AWS_ACCESS_KEY_ID = '__internal_call__'
+INTERNAL_AWS_SECRET_ACCESS_KEY = '__internal_call__'
