@@ -31,6 +31,7 @@ def do_register_localstack_plugins():
     try:
         from localstack.services import edge
         from localstack.services.apigateway import apigateway_starter
+        from localstack.services.cloudformation import cloudformation_starter, cloudformation_listener
         from localstack.services.s3 import s3_listener, s3_starter
         from localstack.services.ec2 import ec2_starter, ec2_listener
         from localstack.services.kms import kms_starter
@@ -44,7 +45,7 @@ def do_register_localstack_plugins():
         from localstack.services.logs import logs_listener, logs_starter
         from localstack.services.infra import (
             start_sns, start_elasticsearch_service, start_lambda, start_sts, start_ssm,
-            start_redshift, start_firehose, start_dynamodbstreams, start_acm
+            start_redshift, start_firehose, start_dynamodbstreams, start_acm, start_cloudformation
         )
         from localstack.services.events import events_listener, events_starter
         from localstack.services.plugins import Plugin, register_plugin
@@ -52,7 +53,6 @@ def do_register_localstack_plugins():
         from localstack.services.dynamodb import dynamodb_listener, dynamodb_starter
         from localstack.services.apigateway import apigateway_listener
         from localstack.services.stepfunctions import stepfunctions_starter, stepfunctions_listener
-        from localstack.services.cloudformation import cloudformation_listener, cloudformation_starter
         from localstack.services.secretsmanager import secretsmanager_listener
         from localstack.services.secretsmanager import secretsmanager_starter
         from localstack.services.cloudwatch import cloudwatch_listener, cloudwatch_starter
@@ -71,10 +71,18 @@ def do_register_localstack_plugins():
             start=apigateway_starter.start_apigateway,
             listener=apigateway_listener.UPDATE_APIGATEWAY))
 
-        register_plugin(Plugin(
-            'cloudformation',
-            start=cloudformation_starter.start_cloudformation,
-            listener=cloudformation_listener.UPDATE_CLOUDFORMATION))
+        if config.USE_MOTO_CF:
+            # TODO: deprecated - remove in a future iteration
+            register_plugin(Plugin(
+                'cloudformation',
+                start=cloudformation_starter.start_cloudformation,
+                listener=cloudformation_listener.UPDATE_CLOUDFORMATION
+            ))
+        else:
+            register_plugin(Plugin(
+                'cloudformation',
+                start=start_cloudformation
+            ))
 
         register_plugin(Plugin(
             'cloudwatch',
