@@ -19,6 +19,10 @@ from flask import Flask, Response, jsonify, request
 from localstack import config
 from localstack.constants import TEST_AWS_ACCOUNT_ID
 from localstack.utils.aws import aws_stack, aws_responses
+from localstack.utils.common import (
+    to_str, to_bytes, load_file, save_file, TMP_FILES, ensure_readable, short_uid, long_uid, json_safe,
+    mkdir, unzip, is_zip_file, run, first_char_to_lower, run_for_max_seconds,
+    timestamp_millis, now_utc, safe_requests, FuncThread, isoformat_milliseconds, synchronized)
 from localstack.services.awslambda import lambda_executors
 from localstack.services.awslambda.lambda_executors import (
     LAMBDA_RUNTIME_PYTHON27,
@@ -37,14 +41,11 @@ from localstack.services.awslambda.lambda_executors import (
     LAMBDA_RUNTIME_RUBY,
     LAMBDA_RUNTIME_RUBY25,
     LAMBDA_RUNTIME_PROVIDED)
-from localstack.services.awslambda.multivalue_transformer import multi_value_dict_for_list
-from localstack.utils.common import (
-    to_str, to_bytes, load_file, save_file, TMP_FILES, ensure_readable, short_uid, json_safe,
-    mkdir, unzip, is_zip_file, run, first_char_to_lower,
-    timestamp_millis, now_utc, safe_requests, FuncThread, isoformat_milliseconds, synchronized)
 from localstack.utils.analytics import event_publisher
 from localstack.utils.http_utils import parse_chunked_data
-from localstack.utils.aws.aws_models import LambdaFunction
+from localstack.utils.aws.aws_models import LambdaFunction, CodeSigningConfig
+from localstack.services.cloudformation.service_models import LAMBDA_POLICY_NAME_PATTERN
+from localstack.services.awslambda.multivalue_transformer import multi_value_dict_for_list
 
 # logger
 LOG = logging.getLogger(__name__)
@@ -87,6 +88,9 @@ app = Flask(APP_NAME)
 # map ARN strings to lambda function objects
 ARN_TO_LAMBDA = {}
 
+# map ARN strigns to CodeSigningConfig object
+ARN_TO_CODE_SIGNING_CONFIG = {}
+
 # list of event source mappings for the API
 EVENT_SOURCE_MAPPINGS = []
 
@@ -121,7 +125,6 @@ LAMBDA_EXECUTOR = lambda_executors.AVAILABLE_EXECUTORS.get(config.LAMBDA_EXECUTO
 
 # IAM policy constants
 IAM_POLICY_VERSION = '2012-10-17'
-POLICY_NAME_PATTERN = 'lambda_policy_%s'
 
 # Whether to check if the handler function exists while creating lambda function
 CHECK_HANDLER_ON_CREATION = False
@@ -170,6 +173,16 @@ def func_arn(function_name):
     return aws_stack.lambda_function_arn(function_name)
 
 
+def func_qualifier(function_name, qualifier=None):
+    arn = aws_stack.lambda_function_arn(function_name)
+    details = ARN_TO_LAMBDA.get(arn)
+    if not details:
+        return details
+    if details.qualifier_exists(qualifier):
+        return '{}:{}'.format(arn, qualifier)
+    return arn
+
+
 def check_batch_size_range(source_arn, batch_size=None):
     batch_size_entry = BATCH_SIZE_RANGES.get(source_arn.split(':')[2].lower())
     if not batch_size_entry:
@@ -296,8 +309,9 @@ def process_apigateway_invocation(func_arn, path, payload, stage, api_id, header
             'stageVariables': get_stage_variables(api_id, stage),
         }
         LOG.debug('Running Lambda function %s from API Gateway invocation: %s %s' % (func_arn, method or 'GET', path))
-        return run_lambda(event=event, context=event_context, func_arn=func_arn,
-            asynchronous=not config.SYNCHRONOUS_API_GATEWAY_EVENTS)
+        asynchronous = not config.SYNCHRONOUS_API_GATEWAY_EVENTS
+        inv_result = run_lambda(event=event, context=event_context, func_arn=func_arn, asynchronous=asynchronous)
+        return inv_result.result
     except Exception as e:
         LOG.warning('Unable to run Lambda function on API Gateway message: %s %s' % (e, traceback.format_exc()))
 
@@ -326,7 +340,8 @@ def process_sns_notification(func_arn, topic_arn, subscription_arn, message, mes
             }
         }]
     }
-    return run_lambda(event=event, context={}, func_arn=func_arn, asynchronous=not config.SYNCHRONOUS_SNS_EVENTS)
+    inv_result = run_lambda(event=event, context={}, func_arn=func_arn, asynchronous=not config.SYNCHRONOUS_SNS_EVENTS)
+    return inv_result.result
 
 
 def process_kinesis_records(records, stream_name):
@@ -496,14 +511,21 @@ def get_function_version(arn, version):
 def publish_new_function_version(arn):
     func_details = ARN_TO_LAMBDA.get(arn)
     versions = func_details.versions
-    last_version = func_details.max_version()
-    versions[str(last_version + 1)] = {
-        'CodeSize': versions.get(VERSION_LATEST).get('CodeSize'),
-        'CodeSha256': versions.get(VERSION_LATEST).get('CodeSha256'),
-        'Function': versions.get(VERSION_LATEST).get('Function'),
-        'RevisionId': str(uuid.uuid4())
-    }
-    return get_function_version(arn, str(last_version + 1))
+    max_version_number = func_details.max_version()
+    next_version_number = max_version_number + 1
+    latest_hash = versions.get(VERSION_LATEST).get('CodeSha256')
+    max_version = versions.get(str(max_version_number))
+    max_version_hash = max_version.get('CodeSha256') if max_version else ''
+
+    if latest_hash != max_version_hash:
+        versions[str(next_version_number)] = {
+            'CodeSize': versions.get(VERSION_LATEST).get('CodeSize'),
+            'CodeSha256': versions.get(VERSION_LATEST).get('CodeSha256'),
+            'Function': versions.get(VERSION_LATEST).get('Function'),
+            'RevisionId': str(uuid.uuid4())
+        }
+        max_version_number = next_version_number
+    return get_function_version(arn, str(max_version_number))
 
 
 def do_list_versions(arn):
@@ -534,10 +556,10 @@ def run_lambda(event, context, func_arn, version=None, suppress_output=False, as
         func_arn = aws_stack.fix_arn(func_arn)
         func_details = ARN_TO_LAMBDA.get(func_arn)
         if not func_details:
-            return not_found_error(msg='The resource specified in the request does not exist.')
+            result = not_found_error(msg='The resource specified in the request does not exist.')
+            return lambda_executors.InvocationResult(result)
 
         context = LambdaContext(func_details, version, context)
-
         result = LAMBDA_EXECUTOR.execute(func_arn, func_details, event, context=context,
             version=version, asynchronous=asynchronous, callback=callback)
 
@@ -549,7 +571,8 @@ def run_lambda(event, context, func_arn, version=None, suppress_output=False, as
             'stackTrace': traceback.format_tb(exc_traceback)
         }
         LOG.info('Error executing Lambda function %s: %s %s' % (func_arn, e, traceback.format_exc()))
-        return Response(json.dumps(response), status=500)
+        log_output = e.log_output if isinstance(e, lambda_executors.InvocationException) else ''
+        return lambda_executors.InvocationResult(Response(json.dumps(response), status=500), log_output)
     finally:
         if suppress_output:
             sys.stdout = stdout_
@@ -603,7 +626,6 @@ def exec_lambda_code(script, handler_function='handler', lambda_cwd=None, lambda
 
 
 def get_handler_file_from_name(handler_name, runtime=LAMBDA_DEFAULT_RUNTIME):
-    # TODO: support Java Lambdas in the future
     if runtime.startswith(LAMBDA_RUNTIME_PROVIDED):
         return 'bootstrap'
     delimiter = '.'
@@ -623,15 +645,13 @@ def get_handler_file_from_name(handler_name, runtime=LAMBDA_DEFAULT_RUNTIME):
 
 
 def get_handler_function_from_name(handler_name, runtime=LAMBDA_DEFAULT_RUNTIME):
-    # TODO: support Java Lambdas in the future
     if runtime.startswith(tuple(DOTNET_LAMBDA_RUNTIMES)):
         return handler_name.split(':')[-1]
-    else:
-        return handler_name.split('.')[-1]
+    return handler_name.split('.')[-1]
 
 
 def error_response(msg, code=500, error_type='InternalFailure'):
-    LOG.warning(msg)
+    LOG.info(msg)
     return aws_responses.flask_error_response_json(msg, code=code, error_type=error_type)
 
 
@@ -719,6 +739,12 @@ def set_archive_code(code, lambda_name, zip_file_content=None):
 
 
 def set_function_code(code, lambda_name, lambda_cwd=None):
+    # unzipping can take some time - limit the execution time to avoid client/network timeout issues
+    run_for_max_seconds(25, do_set_function_code, code, lambda_name, lambda_cwd=lambda_cwd)
+    return {'FunctionName': lambda_name}
+
+
+def do_set_function_code(code, lambda_name, lambda_cwd=None):
     def generic_handler(event, context):
         raise ClientError(('Unable to find executor for Lambda function "%s". Note that ' +
             'Node.js, Golang, and .Net Core Lambdas currently require LAMBDA_EXECUTOR=docker') % lambda_name)
@@ -850,7 +876,8 @@ def format_func_details(func_details, version=None, always_add_version=False):
         'TracingConfig': {'Mode': 'PassThrough'},
         'RevisionId': func_version.get('RevisionId'),
         'State': 'Active',
-        'LastUpdateStatus': 'Successful'
+        'LastUpdateStatus': 'Successful',
+        'PackageType': func_details.package_type
     }
     if func_details.dead_letter_config:
         result['DeadLetterConfig'] = func_details.dead_letter_config
@@ -890,7 +917,7 @@ def forward_to_fallback_url(func_arn, data):
     raise ClientError('Unexpected value for LAMBDA_FALLBACK_URL: %s' % config.LAMBDA_FALLBACK_URL)
 
 
-def get_lambda_policy(function):
+def get_lambda_policy(function, qualifier=None):
     iam_client = aws_stack.connect_to_service('iam')
     policies = iam_client.list_policies(Scope='Local', MaxItems=500)['Policies']
     docs = []
@@ -908,7 +935,8 @@ def get_lambda_policy(function):
         doc['PolicyArn'] = p['Arn']
         doc['Id'] = 'default'
         docs.append(doc)
-    policy = [d for d in docs if d['Statement'][0]['Resource'] == func_arn(function)]
+    res_qualifier = func_qualifier(function, qualifier)
+    policy = [d for d in docs if d['Statement'][0]['Resource'] == res_qualifier]
     return (policy or [None])[0]
 
 
@@ -968,7 +996,10 @@ def create_function():
         func_details.role = data['Role']
         func_details.kms_key_arn = data.get('KMSKeyArn')
         func_details.memory_size = data.get('MemorySize')
+        func_details.code_signing_config_arn = data.get('CodeSigningConfigArn')
         func_details.code = data['Code']
+        # TODO: support package type 'Image'
+        func_details.package_type = 'Zip'
         func_details.set_dead_letter_config(data)
         result = set_function_code(func_details.code, lambda_name)
         if isinstance(result, Response):
@@ -1073,11 +1104,16 @@ def update_function_code(function):
             - name: 'request'
               in: body
     """
+    arn = func_arn(function)
+    if arn not in ARN_TO_LAMBDA:
+        return error_response('Function not found: %s' %
+                arn, 400, error_type='ResourceNotFoundException')
     data = json.loads(to_str(request.data))
     result = set_function_code(data, function)
-    arn = func_arn(function)
     func_details = ARN_TO_LAMBDA.get(arn)
     result.update(format_func_details(func_details))
+    if data.get('Publish'):
+        result['Version'] = publish_new_function_version(arn)['Version']
     if isinstance(result, Response):
         return result
     return jsonify(result or {})
@@ -1143,6 +1179,17 @@ def update_function_configuration(function):
         lambda_details.envvars = env_vars
     if data.get('Timeout'):
         lambda_details.timeout = data['Timeout']
+    if data.get('Role'):
+        lambda_details.role = data['Role']
+    if data.get('MemorySize'):
+        lambda_details.memory_size = data['MemorySize']
+    if data.get('Description'):
+        lambda_details.description = data['Description']
+    if data.get('VpcConfig'):
+        lambda_details.vpc_config = data['VpcConfig']
+    if data.get('KMSKeyArn'):
+        lambda_details.kms_key_arn = data['KMSKeyArn']
+
     return jsonify(data)
 
 
@@ -1192,6 +1239,7 @@ def add_permission(function):
     action = data.get('Action')
     principal = data.get('Principal')
     sourcearn = data.get('SourceArn')
+    qualifier = request.args.get('Qualifier')
     arn = func_arn(function)
     previous_policy = get_lambda_policy(function)
 
@@ -1204,19 +1252,21 @@ def add_permission(function):
                               '(lambda:[*]|lambda:[a-zA-Z]+|[*])' % action,
                               400, error_type='ValidationException')
 
-    new_policy = generate_policy(sid, action, arn, sourcearn, principal)
+    q_arn = func_qualifier(function, qualifier)
+    new_policy = generate_policy(sid, action, q_arn, sourcearn, principal)
+
     if previous_policy:
         statment_with_sid = next((statement for statement in previous_policy['Statement'] if statement['Sid'] == sid),
             None)
         if statment_with_sid:
             return error_response('The statement id (%s) provided already exists. Please provide a new statement id,'
                         ' or remove the existing statement.' % sid, 400, error_type='ResourceConflictException')
+
         new_policy['Statement'].extend(previous_policy['Statement'])
         iam_client.delete_policy(PolicyArn=previous_policy['PolicyArn'])
 
-    iam_client.create_policy(PolicyName=POLICY_NAME_PATTERN % function,
-                            PolicyDocument=json.dumps(new_policy),
-                            Description='Policy for Lambda function "%s"' % function)
+    iam_client.create_policy(PolicyName=LAMBDA_POLICY_NAME_PATTERN % function,
+        PolicyDocument=json.dumps(new_policy), Description='Policy for Lambda function "%s"' % function)
 
     result = {'Statement': json.dumps(new_policy['Statement'][0])}
     return jsonify(result)
@@ -1241,7 +1291,8 @@ def remove_permission(function, statement):
 
 @app.route('%s/functions/<function>/policy' % PATH_ROOT, methods=['GET'])
 def get_policy(function):
-    policy = get_lambda_policy(function)
+    qualifier = request.args.get('Qualifier')
+    policy = get_lambda_policy(function, qualifier)
     if not policy:
         return error_response('The resource you requested does not exist.',
             404, error_type='ResourceNotFoundException')
@@ -1281,10 +1332,15 @@ def invoke_function(function):
                                       error_type='UnsupportedMediaTypeException')
 
     # Default invocation type is RequestResponse
-    invocation_type = request.environ.get('HTTP_X_AMZ_INVOCATION_TYPE', 'RequestResponse')
+    invocation_type = request.headers.get('X-Amz-Invocation-Type', 'RequestResponse')
+    log_type = request.headers.get('X-Amz-Log-Type')
 
-    def _create_response(result, status_code=200, headers={}):
+    def _create_response(invocation_result, status_code=200, headers={}):
         """ Create the final response for the given invocation result. """
+        if not isinstance(invocation_result, lambda_executors.InvocationResult):
+            invocation_result = lambda_executors.InvocationResult(invocation_result)
+        result = invocation_result.result
+        log_output = invocation_result.log_output
         details = {
             'StatusCode': status_code,
             'Payload': result,
@@ -1310,7 +1366,9 @@ def invoke_function(function):
         # Set error headers
         if details.get('FunctionError'):
             details['Headers']['X-Amz-Function-Error'] = str(details['FunctionError'])
-        details['Headers']['X-Amz-Log-Result'] = base64.b64encode(to_bytes(''))  # TODO add logs!
+        # LogResult contains the last 4KB (~4k characters) of log outputs
+        logs = log_output[-4000:] if log_type == 'Tail' else ''
+        details['Headers']['X-Amz-Log-Result'] = base64.b64encode(to_bytes(logs))
         details['Headers']['X-Amz-Executed-Version'] = str(qualifier or VERSION_LATEST)
         # Construct response object
         response_obj = details['Payload']
@@ -1511,17 +1569,23 @@ def list_aliases(function):
                                       key=lambda x: x['Name'])})
 
 
-@app.route('/<version>/functions/<function>/concurrency', methods=['PUT'])
-def put_concurrency(version, function):
+@app.route('/<version>/functions/<function>/concurrency', methods=['GET', 'PUT', 'DELETE'])
+def function_concurrency(version, function):
     # the version for put_concurrency != PATH_ROOT, at the time of this
     # writing it's: /2017-10-31 for this endpoint
     # https://docs.aws.amazon.com/lambda/latest/dg/API_PutFunctionConcurrency.html
     arn = func_arn(function)
-    data = json.loads(request.data)
     lambda_details = ARN_TO_LAMBDA.get(arn)
     if not lambda_details:
         return not_found_error(arn)
-    lambda_details.concurrency = data
+    if request.method == 'GET':
+        data = lambda_details.concurrency
+    if request.method == 'PUT':
+        data = json.loads(request.data)
+        lambda_details.concurrency = data
+    if request.method == 'DELETE':
+        lambda_details.concurrency = None
+        return Response('', status=204)
     return jsonify(data)
 
 
@@ -1590,7 +1654,7 @@ def put_function_event_invoke_config(function):
                 'Destination': str(response.on_successful_invocation)
             },
             'OnFailure': {
-                'Destination': str(response.dead_letter_config)
+                'Destination': str(response.on_failed_invocation)
             }
         }
     })
@@ -1631,6 +1695,182 @@ def delete_function_event_invoke_config(function):
     return Response('', status=204)
 
 
+@app.route('/2020-06-30/functions/<function>/code-signing-config', methods=['GET'])
+def get_function_code_signing_config(function):
+    function_arn = func_arn(function)
+    if function_arn not in ARN_TO_LAMBDA:
+        msg = 'Function not found: %s' % (function_arn)
+        return error_response(msg, 404, error_type='ResourceNotFoundException')
+    lambda_obj = ARN_TO_LAMBDA[function_arn]
+
+    if not lambda_obj.code_signing_config_arn:
+        arn = None
+        function = None
+    else:
+        arn = lambda_obj.code_signing_config_arn
+
+    result = {
+        'CodeSigningConfigArn': arn,
+        'FunctionName': function
+    }
+    return Response(json.dumps(result), status=200)
+
+
+@app.route('/2020-06-30/functions/<function>/code-signing-config', methods=['PUT'])
+def put_function_code_signing_config(function):
+    data = json.loads(request.data)
+
+    arn = data.get('CodeSigningConfigArn')
+    if arn not in ARN_TO_CODE_SIGNING_CONFIG:
+        msg = """The code signing configuration cannot be found.
+        Check that the provided configuration is not deleted: %s.""" % (arn)
+        return error_response(msg, 404, error_type='CodeSigningConfigNotFoundException')
+
+    function_arn = func_arn(function)
+    if function_arn not in ARN_TO_LAMBDA:
+        msg = 'Function not found: %s' % (function_arn)
+        return error_response(msg, 404, error_type='ResourceNotFoundException')
+    lambda_obj = ARN_TO_LAMBDA[function_arn]
+
+    if data.get('CodeSigningConfigArn'):
+        lambda_obj.code_signing_config_arn = arn
+
+    result = {
+        'CodeSigningConfigArn': arn,
+        'FunctionName': function
+    }
+
+    return Response(json.dumps(result), status=200)
+
+
+@app.route('/2020-06-30/functions/<function>/code-signing-config', methods=['DELETE'])
+def delete_function_code_signing_config(function):
+    function_arn = func_arn(function)
+    if function_arn not in ARN_TO_LAMBDA:
+        msg = 'Function not found: %s' % (function_arn)
+        return error_response(msg, 404, error_type='ResourceNotFoundException')
+
+    lambda_obj = ARN_TO_LAMBDA[function_arn]
+
+    lambda_obj.code_signing_config_arn = None
+
+    return Response('', status=204)
+
+
+@app.route('/2020-04-22/code-signing-configs/', methods=['POST'])
+def create_code_signing_config():
+    data = json.loads(request.data)
+    signing_profile_version_arns = data.get('AllowedPublishers').get('SigningProfileVersionArns')
+
+    code_signing_id = 'csc-%s' % long_uid().replace('-', '')[0:17]
+    arn = aws_stack.code_signing_arn(code_signing_id)
+
+    ARN_TO_CODE_SIGNING_CONFIG[arn] = CodeSigningConfig(arn, code_signing_id, signing_profile_version_arns)
+
+    code_signing_obj = ARN_TO_CODE_SIGNING_CONFIG[arn]
+
+    if data.get('Description'):
+        code_signing_obj.description = data['Description']
+    if data.get('CodeSigningPolicies', {}).get('UntrustedArtifactOnDeployment'):
+        code_signing_obj.untrusted_artifact_on_deployment = data['CodeSigningPolicies']['UntrustedArtifactOnDeployment']
+    code_signing_obj.last_modified = isoformat_milliseconds(datetime.utcnow()) + '+0000'
+
+    result = {
+        'CodeSigningConfig': {
+            'AllowedPublishers': {
+                'SigningProfileVersionArns': code_signing_obj.signing_profile_version_arns
+            },
+            'CodeSigningConfigArn': code_signing_obj.arn,
+            'CodeSigningConfigId': code_signing_obj.id,
+            'CodeSigningPolicies': {
+                'UntrustedArtifactOnDeployment': code_signing_obj.untrusted_artifact_on_deployment
+            },
+            'Description': code_signing_obj.description,
+            'LastModified': code_signing_obj.last_modified
+        }
+    }
+
+    return Response(json.dumps(result), status=201)
+
+
+@app.route('/2020-04-22/code-signing-configs/<arn>', methods=['GET'])
+def get_code_signing_config(arn):
+    try:
+        code_signing_obj = ARN_TO_CODE_SIGNING_CONFIG[arn]
+    except KeyError:
+        msg = 'The Lambda code signing configuration %s can not be found.' % arn
+        return error_response(msg, 404, error_type='ResourceNotFoundException')
+
+    result = {
+        'CodeSigningConfig': {
+            'AllowedPublishers': {
+                'SigningProfileVersionArns': code_signing_obj.signing_profile_version_arns
+            },
+            'CodeSigningConfigArn': code_signing_obj.arn,
+            'CodeSigningConfigId': code_signing_obj.id,
+            'CodeSigningPolicies': {
+                'UntrustedArtifactOnDeployment': code_signing_obj.untrusted_artifact_on_deployment
+            },
+            'Description': code_signing_obj.description,
+            'LastModified': code_signing_obj.last_modified
+        }
+    }
+
+    return Response(json.dumps(result), status=200)
+
+
+@app.route('/2020-04-22/code-signing-configs/<arn>', methods=['DELETE'])
+def delete_code_signing_config(arn):
+    try:
+        ARN_TO_CODE_SIGNING_CONFIG.pop(arn)
+    except KeyError:
+        msg = 'The Lambda code signing configuration %s can not be found.' % (arn)
+        return error_response(msg, 404, error_type='ResourceNotFoundException')
+
+    return Response('', status=204)
+
+
+@app.route('/2020-04-22/code-signing-configs/<arn>', methods=['PUT'])
+def update_code_signing_config(arn):
+    try:
+        code_signing_obj = ARN_TO_CODE_SIGNING_CONFIG[arn]
+    except KeyError:
+        msg = 'The Lambda code signing configuration %s can not be found.' % (arn)
+        return error_response(msg, 404, error_type='ResourceNotFoundException')
+
+    data = json.loads(request.data)
+    is_updated = False
+    if data.get('Description'):
+        code_signing_obj.description = data['Description']
+        is_updated = True
+    if data.get('AllowedPublishers', {}).get('SigningProfileVersionArns'):
+        code_signing_obj.signing_profile_version_arns = data['AllowedPublishers']['SigningProfileVersionArns']
+        is_updated = True
+    if data.get('CodeSigningPolicies', {}).get('UntrustedArtifactOnDeployment'):
+        code_signing_obj.untrusted_artifact_on_deployment = data['CodeSigningPolicies']['UntrustedArtifactOnDeployment']
+        is_updated = True
+
+    if is_updated:
+        code_signing_obj.last_modified = isoformat_milliseconds(datetime.utcnow()) + '+0000'
+
+    result = {
+        'CodeSigningConfig': {
+            'AllowedPublishers': {
+                'SigningProfileVersionArns': code_signing_obj.signing_profile_version_arns
+            },
+            'CodeSigningConfigArn': code_signing_obj.arn,
+            'CodeSigningConfigId': code_signing_obj.id,
+            'CodeSigningPolicies': {
+                'UntrustedArtifactOnDeployment': code_signing_obj.untrusted_artifact_on_deployment
+            },
+            'Description': code_signing_obj.description,
+            'LastModified': code_signing_obj.last_modified
+        }
+    }
+
+    return Response(json.dumps(result), status=200)
+
+
 def serve(port, quiet=True):
     from localstack.services import generic_proxy  # moved here to fix circular import errors
 
