@@ -3,16 +3,25 @@ import json
 import random
 import logging
 import threading
+import time
 from binascii import crc32
+from cachetools import TTLCache
 from requests.models import Request, Response
 from localstack import config
 from localstack.utils.aws import aws_stack, aws_responses
-from localstack.utils.common import to_bytes, to_str, clone
+from localstack.utils.common import to_bytes, to_str, clone, select_attributes
 from localstack.utils.analytics import event_publisher
+from localstack.utils.bootstrap import is_api_enabled
 from localstack.services.awslambda import lambda_api
 from localstack.services.generic_proxy import ProxyListener
 from localstack.services.dynamodbstreams import dynamodbstreams_api
 
+# set up logger
+LOGGER = logging.getLogger(__name__)
+
+# cache schema definitions
+SCHEMA_CACHE = TTLCache(maxsize=50, ttl=20)
+
 # cache table definitions - used for testing
 TABLE_DEFINITIONS = {}
 
@@ -22,14 +31,17 @@ TABLE_TAGS = {}
 # action header prefix
 ACTION_PREFIX = 'DynamoDB_20120810'
 
-# set up logger
-LOGGER = logging.getLogger(__name__)
+# maps global table names to configurations
+GLOBAL_TABLES = {}
 
 # list of actions subject to throughput limitations
-THROTTLED_ACTIONS = [
-    'PutItem', 'BatchWriteItem', 'UpdateItem', 'DeleteItem', 'TransactWriteItems',
+READ_THROTTLED_ACTIONS = [
     'GetItem', 'Query', 'Scan', 'TransactGetItems', 'BatchGetItem'
 ]
+WRITE_THROTTLED_ACTIONS = [
+    'PutItem', 'BatchWriteItem', 'UpdateItem', 'DeleteItem', 'TransactWriteItems',
+]
+THROTTLED_ACTIONS = READ_THROTTLED_ACTIONS + WRITE_THROTTLED_ACTIONS
 
 
 class ProxyListenerDynamoDB(ProxyListener):
@@ -38,38 +50,78 @@ class ProxyListenerDynamoDB(ProxyListener):
     def __init__(self):
         self._table_ttl_map = {}
 
-    def forward_request(self, method, path, data, headers):
-        if path.startswith('/shell') or method == 'GET':
-            if path == '/shell':
-                headers = {'Refresh': '0; url=%s/shell/' % config.TEST_DYNAMODB_URL}
-                return aws_responses.requests_response('', headers=headers)
+    @staticmethod
+    def table_exists(ddb_client, table_name):
+        paginator = ddb_client.get_paginator('list_tables')
+        pages = paginator.paginate(
+            PaginationConfig={
+                'PageSize': 100
+            }
+        )
+        for page in pages:
+            table_names = page['TableNames']
+            if to_str(table_name) in table_names:
+                return True
+        return False
+
+    def action_should_throttle(self, action, actions):
+        throttled = ['%s.%s' % (ACTION_PREFIX, a) for a in actions]
+        return action in throttled
+
+    def should_throttle(self, action):
+        rand = random.random()
+        if (rand < config.DYNAMODB_READ_ERROR_PROBABILITY and
+                self.action_should_throttle(action, READ_THROTTLED_ACTIONS)):
             return True
+        elif (rand < config.DYNAMODB_WRITE_ERROR_PROBABILITY and
+                self.action_should_throttle(action, WRITE_THROTTLED_ACTIONS)):
+            return True
+        elif (rand < config.DYNAMODB_ERROR_PROBABILITY and
+                self.action_should_throttle(action, THROTTLED_ACTIONS)):
+            return True
+        else:
+            return False
 
-        if method == 'OPTIONS':
-            return 200
+    def forward_request(self, method, path, data, headers):
+        result = handle_special_request(method, path, data, headers)
+        if result is not None:
+            return result
 
+        if not data:
+            data = '{}'
         data = json.loads(to_str(data))
         ddb_client = aws_stack.connect_to_service('dynamodb')
         action = headers.get('X-Amz-Target')
 
-        if random.random() < config.DYNAMODB_ERROR_PROBABILITY:
-            throttled = ['%s.%s' % (ACTION_PREFIX, a) for a in THROTTLED_ACTIONS]
-            if action in throttled:
-                return error_response_throughput()
+        if self.should_throttle(action):
+            return self.error_response_throughput()
 
         ProxyListenerDynamoDB.thread_local.existing_item = None
 
         if action == '%s.CreateTable' % ACTION_PREFIX:
             # Check if table exists, to avoid error log output from DynamoDBLocal
-            table_names = ddb_client.list_tables()['TableNames']
-            if to_str(data['TableName']) in table_names:
-                return 200
+            if self.table_exists(ddb_client, data['TableName']):
+                return error_response(message='Table already created',
+                                      error_type='ResourceInUseException', code=400)
+
+        if action == '%s.CreateGlobalTable' % ACTION_PREFIX:
+            return create_global_table(data)
+
+        elif action == '%s.DescribeGlobalTable' % ACTION_PREFIX:
+            return describe_global_table(data)
+
+        elif action == '%s.ListGlobalTables' % ACTION_PREFIX:
+            return list_global_tables(data)
+
+        elif action == '%s.UpdateGlobalTable' % ACTION_PREFIX:
+            return update_global_table(data)
 
         elif action in ('%s.PutItem' % ACTION_PREFIX, '%s.UpdateItem' % ACTION_PREFIX, '%s.DeleteItem' % ACTION_PREFIX):
             # find an existing item and store it in a thread-local, so we can access it in return_response,
             # in order to determine whether an item already existed (MODIFY) or not (INSERT)
             try:
-                ProxyListenerDynamoDB.thread_local.existing_item = find_existing_item(data)
+                if has_event_sources_or_streams_enabled(data['TableName']):
+                    ProxyListenerDynamoDB.thread_local.existing_item = find_existing_item(data)
             except Exception as e:
                 if 'ResourceNotFoundException' in str(e):
                     return get_table_not_found_error()
@@ -77,20 +129,19 @@ class ProxyListenerDynamoDB(ProxyListener):
 
             # Fix incorrect values if ReturnValues==ALL_OLD and ReturnConsumedCapacity is
             # empty, see https://github.com/localstack/localstack/issues/2049
-            if data.get('ReturnValues') == 'ALL_OLD' and not data.get('ReturnConsumedCapacity'):
+            if ((data.get('ReturnValues') == 'ALL_OLD') or (not data.get('ReturnValues'))) \
+                    and not data.get('ReturnConsumedCapacity'):
                 data['ReturnConsumedCapacity'] = 'TOTAL'
                 return Request(data=json.dumps(data), method=method, headers=headers)
 
         elif action == '%s.DescribeTable' % ACTION_PREFIX:
             # Check if table exists, to avoid error log output from DynamoDBLocal
-            table_names = ddb_client.list_tables()['TableNames']
-            if to_str(data['TableName']) not in table_names:
+            if not self.table_exists(ddb_client, data['TableName']):
                 return get_table_not_found_error()
 
         elif action == '%s.DeleteTable' % ACTION_PREFIX:
             # Check if table exists, to avoid error log output from DynamoDBLocal
-            table_names = ddb_client.list_tables()['TableNames']
-            if to_str(data['TableName']) not in table_names:
+            if not self.table_exists(ddb_client, data['TableName']):
                 return get_table_not_found_error()
 
         elif action == '%s.BatchWriteItem' % ACTION_PREFIX:
@@ -103,6 +154,14 @@ class ProxyListenerDynamoDB(ProxyListener):
                             existing_items.append(find_existing_item(inner_request, table_name))
             ProxyListenerDynamoDB.thread_local.existing_items = existing_items
 
+        elif action == '%s.Query' % ACTION_PREFIX:
+            if data.get('IndexName'):
+                if not is_index_query_valid(to_str(data['TableName']), data.get('Select')):
+                    return error_response(message='One or more parameter values were invalid: Select type '
+                                                  'ALL_ATTRIBUTES is not supported for global secondary index id-index '
+                                                  'because its projection type is not ALL',
+                                          error_type='ValidationException', code=400)
+
         elif action == '%s.TransactWriteItems' % ACTION_PREFIX:
             existing_items = []
             for item in data['TransactItems']:
@@ -190,10 +249,12 @@ class ProxyListenerDynamoDB(ProxyListener):
         if not action:
             return
 
+        # upgrade event version to 1.1
         record = {
             'eventID': '1',
-            'eventVersion': '1.0',
+            'eventVersion': '1.1',
             'dynamodb': {
+                'ApproximateCreationDateTime': time.time(),
                 'StreamViewType': 'NEW_AND_OLD_IMAGES',
                 'SizeBytes': -1
             },
@@ -202,39 +263,53 @@ class ProxyListenerDynamoDB(ProxyListener):
         }
         records = [record]
 
+        streams_enabled_cache = {}
+        table_name = data.get('TableName')
+        event_sources_or_streams_enabled = has_event_sources_or_streams_enabled(table_name, streams_enabled_cache)
+
         if action == '%s.UpdateItem' % ACTION_PREFIX:
-            if response.status_code == 200:
+            if response.status_code == 200 and event_sources_or_streams_enabled:
+                existing_item = self._thread_local('existing_item')
+                record['eventName'] = 'INSERT' if not existing_item else 'MODIFY'
+
                 updated_item = find_existing_item(data)
                 if not updated_item:
                     return
-                record['eventName'] = 'MODIFY'
                 record['dynamodb']['Keys'] = data['Key']
-                record['dynamodb']['OldImage'] = self._thread_local('existing_item')
+                if existing_item:
+                    record['dynamodb']['OldImage'] = existing_item
                 record['dynamodb']['NewImage'] = updated_item
                 record['dynamodb']['SizeBytes'] = len(json.dumps(updated_item))
         elif action == '%s.BatchWriteItem' % ACTION_PREFIX:
             records = self.prepare_batch_write_item_records(record, data)
+            for record in records:
+                event_sources_or_streams_enabled = (event_sources_or_streams_enabled or
+                    has_event_sources_or_streams_enabled(record['eventSourceARN'], streams_enabled_cache))
 
         elif action == '%s.TransactWriteItems' % ACTION_PREFIX:
             records = self.prepare_transact_write_item_records(record, data)
+            for record in records:
+                event_sources_or_streams_enabled = (event_sources_or_streams_enabled or
+                    has_event_sources_or_streams_enabled(record['eventSourceARN'], streams_enabled_cache))
 
         elif action == '%s.PutItem' % ACTION_PREFIX:
             if response.status_code == 200:
-                existing_item = self._thread_local('existing_item')
-                record['eventName'] = 'INSERT' if not existing_item else 'MODIFY'
-                keys = dynamodb_extract_keys(item=data['Item'], table_name=data['TableName'])
+                keys = dynamodb_extract_keys(item=data['Item'], table_name=table_name)
                 if isinstance(keys, Response):
                     return keys
                 # fix response
                 if response._content == '{}':
-                    response._content = json.dumps({'Attributes': data['Item']})
+                    response._content = update_put_item_response_content(data, response._content)
                     fix_headers_for_updated_response(response)
-                # prepare record keys
-                record['dynamodb']['Keys'] = keys
-                record['dynamodb']['NewImage'] = data['Item']
-                record['dynamodb']['SizeBytes'] = len(json.dumps(data['Item']))
-                if existing_item:
-                    record['dynamodb']['OldImage'] = existing_item
+                if event_sources_or_streams_enabled:
+                    existing_item = self._thread_local('existing_item')
+                    record['eventName'] = 'INSERT' if not existing_item else 'MODIFY'
+                    # prepare record keys
+                    record['dynamodb']['Keys'] = keys
+                    record['dynamodb']['NewImage'] = data['Item']
+                    record['dynamodb']['SizeBytes'] = len(json.dumps(data['Item']))
+                    if existing_item:
+                        record['dynamodb']['OldImage'] = existing_item
 
         elif action in ['%s.GetItem' % ACTION_PREFIX, '%s.Query' % ACTION_PREFIX]:
             if response.status_code == 200:
@@ -243,7 +318,7 @@ class ProxyListenerDynamoDB(ProxyListener):
                 # returned by dynalite, but not by AWS's DynamoDBLocal
                 if 'ConsumedCapacity' not in content and data.get('ReturnConsumedCapacity') in ['TOTAL', 'INDEXES']:
                     content['ConsumedCapacity'] = {
-                        'TableName': data['TableName'],
+                        'TableName': table_name,
                         'CapacityUnits': 5,             # TODO hardcoded
                         'ReadCapacityUnits': 2,
                         'WriteCapacityUnits': 3
@@ -252,7 +327,7 @@ class ProxyListenerDynamoDB(ProxyListener):
                     fix_headers_for_updated_response(response)
 
         elif action == '%s.DeleteItem' % ACTION_PREFIX:
-            if response.status_code == 200:
+            if response.status_code == 200 and event_sources_or_streams_enabled:
                 old_item = self._thread_local('existing_item')
                 record['eventName'] = 'REMOVE'
                 record['dynamodb']['Keys'] = data['Key']
@@ -265,22 +340,23 @@ class ProxyListenerDynamoDB(ProxyListener):
                     create_dynamodb_stream(data, content['TableDescription'].get('LatestStreamLabel'))
 
             event_publisher.fire_event(event_publisher.EVENT_DYNAMODB_CREATE_TABLE,
-                payload={'n': event_publisher.get_hash(data['TableName'])})
+                payload={'n': event_publisher.get_hash(table_name)})
 
-            if data.get('Tags'):
+            if data.get('Tags') and response.status_code == 200:
                 table_arn = json.loads(response._content)['TableDescription']['TableArn']
                 TABLE_TAGS[table_arn] = {tag['Key']: tag['Value'] for tag in data['Tags']}
 
             return
 
         elif action == '%s.DeleteTable' % ACTION_PREFIX:
+            table_arn = json.loads(response._content).get('TableDescription', {}).get('TableArn')
             event_publisher.fire_event(
                 event_publisher.EVENT_DYNAMODB_DELETE_TABLE,
-                payload={'n': event_publisher.get_hash(data['TableName'])}
+                payload={'n': event_publisher.get_hash(table_name)}
             )
-
-            TABLE_TAGS.pop(json.loads(response._content).get('TableDescription', {}).get('TableArn', None), None)
-
+            self.delete_all_event_source_mappings(table_arn)
+            dynamodbstreams_api.delete_streams(table_arn)
+            TABLE_TAGS.pop(table_arn, None)
             return
 
         elif action == '%s.UpdateTable' % ACTION_PREFIX:
@@ -294,29 +370,30 @@ class ProxyListenerDynamoDB(ProxyListener):
             table_arn = data['ResourceArn']
             if table_arn not in TABLE_TAGS:
                 TABLE_TAGS[table_arn] = {}
-
             TABLE_TAGS[table_arn].update({tag['Key']: tag['Value'] for tag in data.get('Tags', [])})
-
             return
 
         elif action == '%s.UntagResource' % ACTION_PREFIX:
             table_arn = data['ResourceArn']
             for tag_key in data.get('TagKeys', []):
                 TABLE_TAGS.get(table_arn, {}).pop(tag_key, None)
-
             return
 
         else:
             # nothing to do
             return
 
-        if len(records) > 0 and 'eventName' in records[0]:
+        if event_sources_or_streams_enabled and records and 'eventName' in records[0]:
             if 'TableName' in data:
-                records[0]['eventSourceARN'] = aws_stack.dynamodb_table_arn(data['TableName'])
+                records[0]['eventSourceARN'] = aws_stack.dynamodb_table_arn(table_name)
 
             forward_to_lambda(records)
             forward_to_ddb_stream(records)
 
+    # -------------
+    # UTIL METHODS
+    # -------------
+
     def prepare_batch_write_item_records(self, record, data):
         records = []
         i = 0
@@ -352,7 +429,10 @@ class ProxyListenerDynamoDB(ProxyListener):
 
     def prepare_transact_write_item_records(self, record, data):
         records = []
-        for i, request in enumerate(data['TransactItems']):
+        # Fix issue #2745: existing_items only contain the Put/Update/Delete records,
+        # so we will increase the index based on these events
+        i = 0
+        for request in data['TransactItems']:
             put_request = request.get('Put')
             if put_request:
                 existing_item = self._thread_local('existing_items')[i]
@@ -368,6 +448,7 @@ class ProxyListenerDynamoDB(ProxyListener):
                     new_record['dynamodb']['OldImage'] = existing_item
                 new_record['eventSourceARN'] = aws_stack.dynamodb_table_arn(table_name)
                 records.append(new_record)
+                i += 1
             update_request = request.get('Update')
             if update_request:
                 table_name = update_request['TableName']
@@ -384,6 +465,7 @@ class ProxyListenerDynamoDB(ProxyListener):
                 new_record['dynamodb']['NewImage'] = updated_item
                 new_record['eventSourceARN'] = aws_stack.dynamodb_table_arn(table_name)
                 records.append(new_record)
+                i += 1
             delete_request = request.get('Delete')
             if delete_request:
                 table_name = delete_request['TableName']
@@ -396,8 +478,21 @@ class ProxyListenerDynamoDB(ProxyListener):
                 new_record['dynamodb']['OldImage'] = self._thread_local('existing_items')[i]
                 new_record['eventSourceARN'] = aws_stack.dynamodb_table_arn(table_name)
                 records.append(new_record)
+                i += 1
         return records
 
+    def delete_all_event_source_mappings(self, table_arn):
+        if table_arn:
+            # fix start dynamodb service without lambda
+            if not is_api_enabled('lambda'):
+                return
+
+            lambda_client = aws_stack.connect_to_service('lambda')
+            result = lambda_client.list_event_source_mappings(EventSourceArn=table_arn)
+            for event in result['EventSourceMappings']:
+                event_source_mapping_id = event['UUID']
+                lambda_client.delete_event_source_mapping(UUID=event_source_mapping_id)
+
     @staticmethod
     def _thread_local(name, default=None):
         try:
@@ -406,6 +501,106 @@ class ProxyListenerDynamoDB(ProxyListener):
             return default
 
 
+def handle_special_request(method, path, data, headers):
+    if path.startswith('/shell') or method == 'GET':
+        if path == '/shell':
+            headers = {'Refresh': '0; url=%s/shell/' % config.TEST_DYNAMODB_URL}
+            return aws_responses.requests_response('', headers=headers)
+        return True
+
+    if method == 'OPTIONS':
+        return 200
+
+
+def create_global_table(data):
+    table_name = data['GlobalTableName']
+    if table_name in GLOBAL_TABLES:
+        return get_error_message('Global Table with this name already exists', 'GlobalTableAlreadyExistsException')
+    GLOBAL_TABLES[table_name] = data
+    for group in data.get('ReplicationGroup', []):
+        group['ReplicaStatus'] = 'ACTIVE'
+        group['ReplicaStatusDescription'] = 'Replica active'
+    result = {'GlobalTableDescription': data}
+    return result
+
+
+def describe_global_table(data):
+    table_name = data['GlobalTableName']
+    details = GLOBAL_TABLES.get(table_name)
+    if not details:
+        return get_error_message('Global Table with this name does not exist', 'GlobalTableNotFoundException')
+    result = {'GlobalTableDescription': details}
+    return result
+
+
+def list_global_tables(data):
+    result = [select_attributes(tab, ['GlobalTableName', 'ReplicationGroup']) for tab in GLOBAL_TABLES.values()]
+    result = {'GlobalTables': result}
+    return result
+
+
+def update_global_table(data):
+    table_name = data['GlobalTableName']
+    details = GLOBAL_TABLES.get(table_name)
+    if not details:
+        return get_error_message('Global Table with this name does not exist', 'GlobalTableNotFoundException')
+    for update in data.get('ReplicaUpdates', []):
+        repl_group = details['ReplicationGroup']
+        # delete existing
+        delete = update.get('Delete')
+        if delete:
+            details['ReplicationGroup'] = [g for g in repl_group if g['RegionName'] != delete['RegionName']]
+        # create new
+        create = update.get('Create')
+        if create:
+            exists = [g for g in repl_group if g['RegionName'] == create['RegionName']]
+            if exists:
+                continue
+            new_group = {
+                'RegionName': create['RegionName'], 'ReplicaStatus': 'ACTIVE',
+                'ReplicaStatusDescription': 'Replica active'
+            }
+            details['ReplicationGroup'].append(new_group)
+    result = {'GlobalTableDescription': details}
+    return result
+
+
+def is_index_query_valid(table_name, index_query_type):
+    schema = get_table_schema(table_name)
+    for index in schema['Table'].get('GlobalSecondaryIndexes', []):
+        index_projection_type = index.get('Projection').get('ProjectionType')
+        if index_query_type == 'ALL_ATTRIBUTES' and index_projection_type != 'ALL':
+            return False
+    return True
+
+
+def has_event_sources_or_streams_enabled(table_name, cache={}):
+    if not table_name:
+        return
+    table_arn = aws_stack.dynamodb_table_arn(table_name)
+    cached = cache.get(table_arn)
+    if isinstance(cached, bool):
+        return cached
+    sources = lambda_api.get_event_sources(source_arn=table_arn)
+    result = False
+    if sources:
+        result = True
+    if not result and dynamodbstreams_api.get_stream_for_table(table_arn):
+        result = True
+    cache[table_arn] = result
+    return result
+
+
+def get_table_schema(table_name):
+    key = '%s/%s' % (aws_stack.get_region(), table_name)
+    schema = SCHEMA_CACHE.get(key)
+    if not schema:
+        ddb_client = aws_stack.connect_to_service('dynamodb')
+        schema = ddb_client.describe_table(TableName=table_name)
+        SCHEMA_CACHE[key] = schema
+    return schema
+
+
 def find_existing_item(put_item, table_name=None):
     table_name = table_name or put_item['TableName']
     ddb_client = aws_stack.connect_to_service('dynamodb')
@@ -414,9 +609,10 @@ def find_existing_item(put_item, table_name=None):
     if 'Key' in put_item:
         search_key = put_item['Key']
     else:
-        schema = ddb_client.describe_table(TableName=table_name)
+        schema = get_table_schema(table_name)
         schemas = [schema['Table']['KeySchema']]
         for index in schema['Table'].get('GlobalSecondaryIndexes', []):
+            # TODO
             # schemas.append(index['KeySchema'])
             pass
         for schema in schemas:
@@ -428,28 +624,43 @@ def find_existing_item(put_item, table_name=None):
 
     req = {'TableName': table_name, 'Key': search_key}
     existing_item = aws_stack.dynamodb_get_item_raw(req)
+    if not existing_item:
+        return existing_item
     if 'Item' not in existing_item:
         if 'message' in existing_item:
             table_names = ddb_client.list_tables()['TableNames']
-            msg = ('Unable to get item from DynamoDB (existing tables: %s): %s' %
+            msg = ('Unable to get item from DynamoDB (existing tables: %s ...truncated if >100 tables): %s' %
                 (table_names, existing_item['message']))
             LOGGER.warning(msg)
         return
     return existing_item.get('Item')
 
 
-def get_table_not_found_error():
-    response = error_response(message='Cannot do operations on a non-existent table',
-                              error_type='ResourceNotFoundException')
+def get_error_message(message, error_type):
+    response = error_response(message=message, error_type=error_type)
     fix_headers_for_updated_response(response)
     return response
 
 
+def get_table_not_found_error():
+    return get_error_message(message='Cannot do operations on a non-existent table',
+                             error_type='ResourceNotFoundException')
+
+
 def fix_headers_for_updated_response(response):
     response.headers['content-length'] = len(to_bytes(response.content))
     response.headers['x-amz-crc32'] = calculate_crc32(response)
 
 
+def update_put_item_response_content(data, response_content):
+    # when return-values variable is set only then attribute data should be returned
+    # in the response otherwise by default is should not return any data.
+    # https://github.com/localstack/localstack/issues/2121
+    if data.get('ReturnValues'):
+        response_content = json.dumps({'Attributes': data['Item']})
+    return response_content
+
+
 def calculate_crc32(response):
     return crc32(to_bytes(response.content)) & 0xffffffff
 
@@ -477,7 +688,11 @@ def forward_to_lambda(records):
             'Records': [record]
         }
         for src in sources:
-            lambda_api.run_lambda(event=event, context={}, func_arn=src['FunctionArn'])
+            if src.get('State') != 'Enabled':
+                continue
+
+            lambda_api.run_lambda(event=event, context={}, func_arn=src['FunctionArn'],
+                asynchronous=not config.SYNCHRONOUS_DYNAMODB_EVENTS)
 
 
 def forward_to_ddb_stream(records):
