@@ -1,5 +1,4 @@
 import time
-import gzip
 import re
 import json
 import uuid
@@ -13,6 +12,7 @@ import collections
 import dateutil.parser
 import urllib.parse
 import six
+import functools
 import botocore.config
 from pytz import timezone
 from urllib.parse import parse_qs
@@ -25,6 +25,7 @@ from requests.models import Response, Request
 from six.moves.urllib import parse as urlparse
 from localstack import config, constants
 from localstack.config import HOSTNAME, HOSTNAME_EXTERNAL, LOCALHOST_IP
+from localstack.constants import TEST_AWS_ACCESS_KEY_ID, TEST_AWS_SECRET_ACCESS_KEY
 from localstack.utils.aws import aws_stack
 from localstack.services.s3 import multipart_content
 from localstack.utils.common import (
@@ -33,8 +34,8 @@ from localstack.utils.common import (
 from localstack.utils.analytics import event_publisher
 from localstack.utils.http_utils import uses_chunked_encoding
 from localstack.utils.persistence import PersistingProxyListener
-from localstack.constants import TEST_AWS_ACCESS_KEY_ID, TEST_AWS_SECRET_ACCESS_KEY
 from localstack.utils.aws.aws_responses import requests_response, requests_error_response_xml_signature_calculation
+from localstack.services.cloudformation.service_models import S3Bucket
 
 CONTENT_SHA256_HEADER = 'x-amz-content-sha256'
 STREAMING_HMAC_PAYLOAD = 'STREAMING-AWS4-HMAC-SHA256-PAYLOAD'
@@ -89,9 +90,9 @@ ALLOWED_HEADER_OVERRIDES = {
     'response-content-encoding': 'Content-Encoding',
 }
 
-# From botocore's auth.py:
-# https://github.com/boto/botocore/blob/30206ab9e9081c80fa68e8b2cb56296b09be6337/botocore/auth.py#L47
-POLICY_EXPIRATION_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
+# STS policy expiration date format
+POLICY_EXPIRATION_FORMAT1 = '%Y-%m-%dT%H:%M:%SZ'
+POLICY_EXPIRATION_FORMAT2 = '%Y-%m-%dT%H:%M:%S.%IZ'
 
 # ignored_headers_lower conatins headers which don't get involved in signature calculations process
 # these headers are being sent by the localstack by default.
@@ -110,6 +111,8 @@ CORS_HEADERS = [
     'Access-Control-Request-Headers', 'Access-Control-Request-Method'
 ]
 
+SIGNATURE_V4_PARAMS = ['X-Amz-Algorithm']
+
 
 def event_type_matches(events, action, api_method):
     """ check whether any of the event types in `events` matches the
@@ -576,6 +579,57 @@ def fix_delimiter(data, headers, response):
             response._content = re.compile(pattern).sub(delimiter, c)
 
 
+def fix_sorting_versions(method, parsed, response):
+
+    try:
+        if method != 'GET':
+            return
+
+        if not parsed or not parsed.query or 'versions' not in parsed.query:
+            return
+        try:
+            content = to_str(response._content)
+        except Exception:
+            # return in case of conversion errors
+            return
+
+        if not content or '<ListVersionsResult' not in content:
+            return
+
+        parsed = xmltodict.parse(content).get('ListVersionsResult')
+        if not isinstance(parsed.get('Version'), list):
+            return
+
+        result = list(parsed.get('Version'))
+
+        def compare(item1, item2):
+            # sort based on key in ascending order
+            if item1['Key'] < item2['Key']:
+                return -1
+            elif item1['Key'] > item2['Key']:
+                return 1
+            else:
+                # sort based on last modified in descending order
+                t1 = time.mktime(datetime.datetime.strptime(item1['LastModified'], '%Y-%m-%dT%H:%M:%S.%fZ').timetuple())
+                t2 = time.mktime(datetime.datetime.strptime(item2['LastModified'], '%Y-%m-%dT%H:%M:%S.%fZ').timetuple())
+                if t1 > t2:
+                    return -1
+                elif t1 < t2:
+                    return 1
+                else:
+                    # sort based on latest
+                    if item1['IsLatest'] == 'true':
+                        return -1
+                    elif item2['IsLatest'] == 'true':
+                        return 1
+            return 0
+
+        parsed['Version'] = sorted(result, key=functools.cmp_to_key(compare))
+        response._content = xmltodict.unparse({'ListVersionsResult': parsed})
+    except Exception:
+        return
+
+
 def convert_to_chunked_encoding(method, path, response):
     if method != 'GET' or path != '/':
         return
@@ -874,10 +928,7 @@ def is_object_specific_request(path, headers):
 
 
 def normalize_bucket_name(bucket_name):
-    bucket_name = bucket_name or ''
-    # AWS appears to automatically convert upper to lower case chars in bucket names
-    bucket_name = bucket_name.lower()
-    return bucket_name
+    return S3Bucket.normalize_bucket_name(bucket_name)
 
 
 def get_key_name(path, headers):
@@ -890,7 +941,9 @@ def get_key_name(path, headers):
 
 
 def uses_path_addressing(headers):
-    host = headers.get(constants.HEADER_LOCALSTACK_EDGE_URL, '').split('://')[-1] or headers['host']
+    # we can assume that the host header we are receiving here is actually the header we originally recieved
+    # from the client (because the edge service is forwarding the request in memory)
+    host = headers.get('host') or headers.get(constants.HEADER_LOCALSTACK_EDGE_URL, '').split('://')[-1]
     return host.startswith(HOSTNAME) or host.startswith(HOSTNAME_EXTERNAL) or host.startswith(LOCALHOST_IP)
 
 
@@ -1000,6 +1053,14 @@ class ProxyListenerS3(PersistingProxyListener):
     def is_s3_copy_request(headers, path):
         return 'x-amz-copy-source' in headers or 'x-amz-copy-source' in path
 
+    @staticmethod
+    def is_create_multipart_request(query):
+        return query.startswith('uploads')
+
+    @staticmethod
+    def is_multipart_upload(query):
+        return query.startswith('uploadId')
+
     @staticmethod
     def get_201_response(key, bucket_name):
         return """
@@ -1040,11 +1101,20 @@ class ProxyListenerS3(PersistingProxyListener):
         if (method == 'POST' and query.startswith('uploadId')) or contains_cred or contains_key:
             return True
 
+    @staticmethod
+    def parse_policy_expiration_date(expiration_string):
+        try:
+            return datetime.datetime.strptime(expiration_string, POLICY_EXPIRATION_FORMAT1)
+        except Exception:
+            return datetime.datetime.strptime(expiration_string, POLICY_EXPIRATION_FORMAT2)
+
     def forward_request(self, method, path, data, headers):
 
         # Create list of query parameteres from the url
         parsed = urlparse.urlparse('{}{}'.format(config.get_edge_url(), path))
         query_params = parse_qs(parsed.query)
+        path_orig = path
+        path = path.replace('#', '%23')  # support key names containing hashes (e.g., required by Amplify)
 
         # Detecting pre-sign url and checking signature
         if any([p in query_params for p in PRESIGN_QUERY_PARAMS]):
@@ -1060,8 +1130,9 @@ class ProxyListenerS3(PersistingProxyListener):
         if 's3.amazonaws.com' not in headers.get('host', ''):
             headers['host'] = 'localhost'
 
-        # check content md5 hash integrity if not a copy request
-        if 'Content-MD5' in headers and not self.is_s3_copy_request(headers, path):
+        # check content md5 hash integrity if not a copy request or multipart initialization
+        if 'Content-MD5' in headers and not self.is_s3_copy_request(headers, path) \
+                and not self.is_create_multipart_request(parsed_path.query):
             response = check_content_md5(data, headers)
             if response is not None:
                 return response
@@ -1137,7 +1208,7 @@ class ProxyListenerS3(PersistingProxyListener):
                 policy = json.loads(base64.b64decode(policy_value).decode('utf-8'))
                 expiration_string = policy.get('expiration', None)  # Example: 2020-06-05T13:37:12Z
                 if expiration_string:
-                    expiration_datetime = datetime.datetime.strptime(expiration_string, POLICY_EXPIRATION_FORMAT)
+                    expiration_datetime = self.parse_policy_expiration_date(expiration_string)
                     expiration_timestamp = expiration_datetime.timestamp()
                     if is_url_already_expired(expiration_timestamp):
                         return token_expired_error(path, headers.get('x-amz-request-id'), 400)
@@ -1174,11 +1245,12 @@ class ProxyListenerS3(PersistingProxyListener):
             if method == 'PUT':
                 return set_object_lock(bucket, data)
 
-        if modified_data is not None or headers_changed:
+        path_orig_escaped = path_orig.replace('#', '%23')
+        if modified_data is not None or headers_changed or path_orig != path_orig_escaped:
             data_to_return = not_none_or(modified_data, data)
             if modified_data is not None:
                 headers['Content-Length'] = str(len(data_to_return or ''))
-            return Request(data=data_to_return, headers=headers, method=method)
+            return Request(url=path_orig_escaped, data=data_to_return, headers=headers, method=method)
         return True
 
     def get_forward_url(self, method, path, data, headers):
@@ -1197,6 +1269,8 @@ class ProxyListenerS3(PersistingProxyListener):
     def return_response(self, method, path, data, headers, response, request_handler=None):
         path = to_str(path)
         method = to_str(method)
+        path = path.replace('#', '%23')
+
         # persist this API call to disk
         super(ProxyListenerS3, self).return_response(method, path, data, headers, response, request_handler)
 
@@ -1305,6 +1379,7 @@ class ProxyListenerS3(PersistingProxyListener):
             ret304_on_etag(data, headers, response)
             append_aws_request_troubleshooting_headers(response)
             fix_delimiter(data, headers, response)
+            fix_sorting_versions(method, parsed, response)
 
             if method == 'PUT':
                 set_object_expiry(path, headers)
@@ -1375,11 +1450,6 @@ class ProxyListenerS3(PersistingProxyListener):
             # convert to chunked encoding, for compatibility with certain SDKs (e.g., AWS PHP SDK)
             convert_to_chunked_encoding(method, path, response)
 
-            if headers.get('Accept-Encoding') == 'gzip' and response._content:
-                response._content = gzip.compress(to_bytes(response._content))
-                response.headers['Content-Length'] = str(len(response._content))
-                response.headers['Content-Encoding'] = 'gzip'
-
 
 def authenticate_presign_url(method, path, headers, data=None):
 
@@ -1444,6 +1514,13 @@ def authenticate_presign_url(method, path, headers, data=None):
     # Comparing the signature in url with signature we calculated
     query_sig = urlparse.unquote(query_params['Signature'][0])
     if query_sig != signature:
+
+        # older signature calculation methods are not supported
+        # so logging a warning to the user to use the v4 calculation method
+        for param in SIGNATURE_V4_PARAMS:
+            if param.lower() not in (query_param.lower() for query_param in query_params):
+                LOGGER.warning(
+                    'Older version of signature calculation method detected. Please use v4 calculation method')
         return requests_error_response_xml_signature_calculation(
             code=403,
             code_string='SignatureDoesNotMatch',
