@@ -21,6 +21,9 @@ S3_MAX_FILE_SIZE_MB = 2048
 # temporary state
 TMP_STATE = {}
 
+# Key for tracking patch applience
+PATCHES_APPLIED = 'S3_PATCHED'
+
 
 def check_s3(expect_shutdown=False, print_error=False):
     out = None
@@ -56,16 +59,13 @@ def start_s3(port=None, backend_port=None, asynchronous=None, update_listener=No
 
 
 def apply_patches():
+    if TMP_STATE.get(PATCHES_APPLIED, False):
+        return
+
+    TMP_STATE[PATCHES_APPLIED] = True
+
     s3_models.DEFAULT_KEY_BUFFER_SIZE = S3_MAX_FILE_SIZE_MB * 1024 * 1024
 
-    def init(self, name, value, storage='STANDARD', etag=None,
-            is_versioned=False, version_id=0, max_buffer_size=None, *args, **kwargs):
-        return original_init(self, name, value, storage=storage, etag=etag, is_versioned=is_versioned,
-            version_id=version_id, max_buffer_size=s3_models.DEFAULT_KEY_BUFFER_SIZE, *args, **kwargs)
-
-    original_init = s3_models.FakeKey.__init__
-    s3_models.FakeKey.__init__ = init
-
     def s3_update_acls(self, request, query, bucket_name, key_name):
         # fix for - https://github.com/localstack/localstack/issues/1733
         #         - https://github.com/localstack/localstack/issues/1170
@@ -117,6 +117,7 @@ def apply_patches():
     def delete_bucket(self, bucket_name, *args, **kwargs):
         bucket_name = s3_listener.normalize_bucket_name(bucket_name)
         try:
+            s3_listener.remove_bucket_notification(bucket_name)
             return delete_bucket_orig(bucket_name, *args, **kwargs)
         except s3_exceptions.MissingBucket:
             pass
