@@ -2,14 +2,15 @@ import io
 import os
 import re
 import sys
+import glob
 import json
 import uuid
 import time
-import glob
 import base64
 import socket
 import hashlib
 import decimal
+import inspect
 import logging
 import tarfile
 import zipfile
@@ -28,12 +29,13 @@ from datetime import datetime, date
 from contextlib import closing
 from six import with_metaclass
 from six.moves import cStringIO as StringIO
+from six.moves.queue import Queue
 from six.moves.urllib.parse import urlparse, parse_qs
 from multiprocessing.dummy import Pool
 from localstack import config
+from localstack.utils import bootstrap
 from localstack.config import DEFAULT_ENCODING
 from localstack.constants import ENV_DEV
-from localstack.utils import bootstrap
 from localstack.utils.bootstrap import FuncThread
 
 # arrays for temporary files and resources
@@ -78,6 +80,7 @@ class CustomEncoder(json.JSONEncoder):
     """ Helper class to convert JSON documents with datetime, decimals, or bytes. """
 
     def default(self, o):
+        import yaml  # leave import here, to avoid breaking our Lambda tests!
         if isinstance(o, decimal.Decimal):
             if o % 1 > 0:
                 return float(o)
@@ -85,9 +88,17 @@ class CustomEncoder(json.JSONEncoder):
                 return int(o)
         if isinstance(o, (datetime, date)):
             return timestamp_millis(o)
-        if isinstance(o, six.binary_type):
-            return to_str(o)
+        if isinstance(o, yaml.ScalarNode):
+            if o.tag == 'tag:yaml.org,2002:int':
+                return int(o.value)
+            if o.tag == 'tag:yaml.org,2002:float':
+                return float(o.value)
+            if o.tag == 'tag:yaml.org,2002:bool':
+                return bool(o.value)
+            return str(o.value)
         try:
+            if isinstance(o, six.binary_type):
+                return to_str(o)
             return super(CustomEncoder, self).default(o)
         except Exception:
             return None
@@ -231,6 +242,16 @@ class JsonObject(object):
         return self.__str__()
 
 
+class DelSafeDict(dict):
+    """Useful when applying jsonpatch. Use it as follows:
+
+        obj.__dict__ = DelSafeDict(obj.__dict__)
+        apply_patch(obj.__dict__, patch)
+    """
+    def __delitem__(self, key, *args, **kwargs):
+        self[key] = None
+
+
 class CaptureOutput(object):
     """ A context manager that captures stdout/stderr of the current thread. Use it as follows:
 
@@ -328,6 +349,11 @@ def start_worker_thread(method, *args, **kwargs):
     return start_thread(method, *args, _shutdown_hook=False, **kwargs)
 
 
+def empty_context_manager():
+    import contextlib
+    return contextlib.nullcontext()
+
+
 def synchronized(lock=None):
     """
     Synchronization decorator as described in
@@ -342,6 +368,39 @@ def synchronized(lock=None):
     return _decorator
 
 
+def prevent_stack_overflow(match_parameters=False):
+    """ Function decorator to protect a function from stack overflows -
+        raises an exception if a (potential) infinite recursion is detected. """
+    def _decorator(wrapped):
+        @functools.wraps(wrapped)
+        def func(*args, **kwargs):
+            def _matches(frame):
+                if frame.function != wrapped.__name__:
+                    return False
+                frame = frame.frame
+
+                if not match_parameters:
+                    return False
+
+                # construct dict of arguments this stack frame has been called with
+                prev_call_args = {frame.f_code.co_varnames[i]: frame.f_locals[frame.f_code.co_varnames[i]]
+                                  for i in range(frame.f_code.co_argcount)}
+
+                # construct dict of arguments the original function has been called with
+                sig = inspect.signature(wrapped)
+                this_call_args = dict(zip(sig.parameters.keys(), args))
+                this_call_args.update(kwargs)
+
+                return prev_call_args == this_call_args
+
+            matching_frames = [frame[2] for frame in inspect.stack(context=1) if _matches(frame)]
+            if matching_frames:
+                raise RecursionError('(Potential) infinite recursion detected')
+            return wrapped(*args, **kwargs)
+        return func
+    return _decorator
+
+
 def is_string(s, include_unicode=True, exclude_binary=False):
     if isinstance(s, six.binary_type) and exclude_binary:
         return False
@@ -446,6 +505,18 @@ def wait_for_port_open(port, http_path=None, expect_success=True, retries=10, sl
     return retry(check, sleep=sleep_time, retries=retries)
 
 
+def port_can_be_bound(port):
+    """ Return whether a local port can be bound to. Note that this is a stricter check
+        than is_port_open(...) above, as is_port_open() may return False if the port is
+        not accessible (i.e., does not respond), yet cannot be bound to. """
+    try:
+        tcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        tcp.bind(('', port))
+        return True
+    except Exception:
+        return False
+
+
 def get_free_tcp_port(blacklist=None):
     blacklist = blacklist or []
     for i in range(10):
@@ -554,6 +625,22 @@ def recurse_object(obj, func, path=''):
     return obj
 
 
+def keys_to_lower(obj):
+    """ Recursively changes all dict keys to first character lowercase. """
+    def fix_keys(o, **kwargs):
+        if isinstance(o, dict):
+            for k, v in dict(o).items():
+                o.pop(k)
+                o[first_char_to_lower(k)] = v
+        return o
+    result = recurse_object(obj, fix_keys)
+    return result
+
+
+def camel_to_snake_case(string):
+    return re.sub(r'(?<!^)(?=[A-Z])', '_', string).replace('__', '_').lower()
+
+
 def base64_to_hex(b64_string):
     return binascii.hexlify(base64.b64decode(b64_string))
 
@@ -656,7 +743,29 @@ def cp_r(src, dst):
     """Recursively copies file/directory"""
     if os.path.isfile(src):
         return shutil.copy(src, dst)
-    return shutil.copytree(src, dst)
+    return shutil.copytree(src, dst, dirs_exist_ok=True)
+
+
+def disk_usage(path, include_hidden=False):
+    total_size = 0
+    for dirpath, dirnames, filenames in os.walk(path):
+        for f in filenames:
+            fp = os.path.join(dirpath, f)
+            # skip if it is symbolic link
+            if not os.path.islink(fp):
+                total_size += os.path.getsize(fp)
+    return total_size
+
+
+def format_bytes(count, default='n/a'):
+    if not is_number(count) or count < 0:
+        return default
+    cnt = float(count)
+    for unit in ('B', 'KB', 'MB', 'GB', 'TB'):
+        if cnt < 1000:
+            return '%s%s' % (format_number(cnt, decimals=3), unit)
+        cnt = cnt / 1000.0
+    return count
 
 
 def download(url, path, verify_ssl=True):
@@ -664,7 +773,9 @@ def download(url, path, verify_ssl=True):
     # make sure we're creating a new session here to
     # enable parallel file downloads during installation!
     s = requests.Session()
-    r = s.get(url, stream=True, verify=verify_ssl)
+    # Use REQUESTS_CA_BUNDLE path. If it doesn't exist, use the method provided settings.
+    # Note that a value that is not False, will result to True and will get the bundle file.
+    r = s.get(url, stream=True, verify=os.getenv('REQUESTS_CA_BUNDLE', verify_ssl))
     # check status code before attempting to read body
     if r.status_code >= 400:
         raise Exception('Failed to download %s, response code %s' % (url, r.status_code))
@@ -675,13 +786,18 @@ def download(url, path, verify_ssl=True):
             os.makedirs(os.path.dirname(path))
         LOG.debug('Starting download from %s to %s (%s bytes)' % (url, path, r.headers.get('content-length')))
         with open(path, 'wb') as f:
+            iter_length = 0
+            iter_limit = 1000000  # print a log line for every 1MB chunk
             for chunk in r.iter_content(DOWNLOAD_CHUNK_SIZE):
                 total += len(chunk)
+                iter_length += len(chunk)
                 if chunk:  # filter out keep-alive new chunks
                     f.write(chunk)
-                    LOG.debug('Writing %s bytes (total %s) to %s' % (len(chunk), total, path))
                 else:
                     LOG.debug('Empty chunk %s (total %s) from %s' % (chunk, total, url))
+                if iter_length >= iter_limit:
+                    LOG.debug('Written %s bytes (total %s) to %s' % (iter_length, total, path))
+                    iter_length = 0
             f.flush()
             os.fsync(f)
         if os.path.getsize(path) == 0:
@@ -696,13 +812,12 @@ def download(url, path, verify_ssl=True):
 
 def parse_request_data(method, path, data):
     """ Extract request data either from query string (for GET) or request body (for POST). """
-    if method == 'POST':
-        result = parse_qs(to_str(data))
-    elif method == 'GET':
+    result = {}
+    if method in ['POST', 'PUT', 'PATCH']:
+        result = parse_qs(to_str(data or ''))
+    if not result:
         parsed_path = urlparse(path)
         result = parse_qs(parsed_path.query)
-    else:
-        return {}
     result = dict([(k, v[0]) for k, v in result.items()])
     return result
 
@@ -711,6 +826,10 @@ def first_char_to_lower(s):
     return '%s%s' % (s[0].lower(), s[1:])
 
 
+def format_number(number, decimals=2):
+    return ('{0:.%sg}' % decimals).format(number)
+
+
 def is_number(s):
     try:
         float(s)  # for int, long and float
@@ -751,6 +870,14 @@ def get_arch():
     raise Exception('Unable to determine system architecture')
 
 
+def is_command_available(cmd):
+    try:
+        run('which %s' % cmd, print_error=False)
+        return True
+    except Exception:
+        return False
+
+
 def short_uid():
     return str(uuid.uuid4())[0:8]
 
@@ -794,6 +921,14 @@ def extract_jsonpath(value, path):
     return result
 
 
+def assign_to_path(target, path, value):
+    path = path.split('.')
+    for i in range(len(path) - 1):
+        target_new = target[path[i]] = target.get(path[i], {})
+        target = target_new
+    target[path[-1]] = value
+
+
 def save_file(file, content, append=False):
     mode = 'a' if append else 'w+'
     if not isinstance(content, six.string_types):
@@ -938,6 +1073,7 @@ def unzip(path, target_dir, overwrite=True):
     if is_alpine():
         # Running the native command can be an order of magnitude faster in Alpine on Travis-CI
         flags = '-o' if overwrite else ''
+        flags += ' -q'
         return run('cd %s; unzip %s %s' % (target_dir, flags, path))
     try:
         zip_ref = zipfile.ZipFile(path, 'r')
@@ -1138,6 +1274,36 @@ def run_cmd_safe(**kwargs):
     return run_safe(run, print_error=False, **kwargs)
 
 
+def run_for_max_seconds(max_secs, _function, *args, **kwargs):
+    """ Run the given function for a maximum of `max_secs` seconds - continue running
+        in a background thread if the function does not finish in time. """
+    def _worker(*_args):
+        result = None
+        try:
+            result = _function(*args, **kwargs)
+        except Exception as e:
+            result = e
+        result = True if result is None else result
+        q.put(result)
+        return result
+    start = now()
+    q = Queue()
+    start_worker_thread(_worker)
+    for i in range(max_secs * 2):
+        result = None
+        try:
+            result = q.get_nowait()
+        except Exception:
+            pass
+        if result is not None:
+            if isinstance(result, Exception):
+                raise result
+            return result
+        if now() - start >= max_secs:
+            return
+        time.sleep(0.5)
+
+
 def run(cmd, cache_duration_secs=0, **kwargs):
     def do_run(cmd):
         return bootstrap.run(cmd, **kwargs)
